#coding: utf-8

import codecs
from datetime import datetime
import sys
import time
import openpyxl
import os.path


workbooks_and_prefix = {}


def main():
    timestamp_prefix_length = None
    if len(sys.argv) > 2:
        timestamp_prefix_length = int(sys.argv[2])

    source_filename = sys.argv[1]
    prefix = source_filename
    lines = codecs.open(source_filename, 'r', 'utf8').readlines()

    # Encountered input data variations:
    #  2025-04-12T03:34:17 [license_2] License usage: tester=1/10 executor=0/0 variable=0 total=1/10
    #  Apr  8 08:02:58 some text: [squish_floating_license] License usage: tester=2/200 executor=0/200 variable=0 total=2/400

    i = None
    for i, l in enumerate(lines):
        if i % 5000 == 0:
            print(f'\rLines processed: {i}', end='')

        if '] License usage: tester=' not in l:
            continue

        if timestamp_prefix_length is None:
            timestamp = l.split(' [', 1)[0]
        else:
            timestamp = l[:timestamp_prefix_length]

        timestamp = parse_timestamp(timestamp)
        try:
            timestamp = timestamp.isoformat()
        except:
            pass
        license_name = l.split('[', 1)[1].split(']', 1)[0]

        # Get: 'tester=1/10 executor=0/0 variable=0 total=1/10'
        counts = l.split('] License usage: ')[1]

        # Get: ['tester=1/10', 'executor=0/0', 'variable=0', 'total=1/10']
        counts = counts.split(' ')

        # Get relevant values:
        tester = counts[0].split('=')[1].split('/')
        executor = counts[1].split('=')[1].split('/')
        variable = counts[2].split('=')[1]

        # Store values:
        append_row(license_name, prefix, [timestamp, int(tester[1]), int(tester[0]), int(executor[1]), int(executor[0]) + int(variable)])
    print(f'\rLines processed: {i}')

    write_workbooks()


def write_workbooks():
    global workbooks_and_prefix
    for license_name, workbook_and_prefix in workbooks_and_prefix.items():
        wb, prefix = workbook_and_prefix
        fn = f'{os.path.basename(prefix)}.{license_name}.xlsx'
        for i in range(5):
            print(f'Writing: {fn}', end='')
            sys.stdout.flush()
            try:
                wb.save(fn)
                break
            except PermissionError as e:
                print()
                print(f'Error while writing the file - is it perhaps still open? Retrying in 3 seconds. ({i+1}/5)')
                if i < 3:
                    time.sleep(3)
        print()


def append_row(license_name, prefix, row):
    wb = get_license_workbook(license_name, prefix)
    wb.active.append(row)


def get_license_workbook(license_name, prefix):
    global workbooks_and_prefix
    if license_name not in workbooks_and_prefix:
        wb = openpyxl.Workbook()
        wb.active.append(['Timestamp', 'Testers in License', 'Used Tester', 'Executions in License', 'Used Executions'])
        workbooks_and_prefix[license_name] = (wb, prefix)
    return workbooks_and_prefix[license_name][0]


def parse_timestamp(timestamp):
    formats = [
        "%Y-%m-%dT%H:%M:%S",
        "%Y-%m-%d %H:%M:%S",
        "%d/%m/%Y %H:%M:%S",
        "%m/%d/%Y %H:%M:%S",
        "%Y-%m-%d",
        "%d/%m/%Y",
        "%m/%d/%Y",
        "%b %d %H:%M:%S"
    ]
    
    for fmt in formats:
        try:
            ts = datetime.strptime(timestamp, fmt)
            # If year is 1900, assume the year was missing from the
            # timestamp, and that it is the current year:
            if ts.year == 1900:
                ts = ts.replace(year=datetime.now().year)
            return ts
        except ValueError:
            continue
    return timestamp


def print_usage():
    print(f'USAGE: {sys.argv[0]} squish_license_server_log_file_here [timestamp_prefix_length]')
    print()
    print('Writes license usage information into new files in the current')
    print('working directory.')
    print()
    print('The file format is CSV (comma separated values).')
    print()
    print('After importing such a file into a spreadsheet application, a line')
    print('chart or similar can be added to get a visual representation of the')
    print('license usage (peak, low, etc.).')
    print()
    print('Also see https://doc.qt.io/squish/license-server.html#license-server-logging')
    print()
    print('timestamp_prefix_length - The length of the timestamp at the beginning of the line.')
    print('                          Without this extracting and parsing the timestamp may fail.')


if __name__ == '__main__':
    if len(sys.argv) < 2:
        print_usage()
        sys.exit(1)
    main()
